package manager;
import java.util.Vector;

/**
 * This class is responsible to manage a Board for any kind of ({@link Game}).
 * The public interface is responsible for all allowed actions in a Board. The game is responsible for check if an action  is valid (in terms of game rules).
 * Each Board contains its pieces ({@link Piece}) and no cell can be null; there is a special kind of piece used in "empty cell": {@link NullPiece}.
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * @see Game
 * @see Piece
 * @see NullPiece
 */
public class Board {
	private Vector BOARD = null;
	private int DIMENSION_X = 0;
	private int DIMENSION_Y = 0;
	
    /*
	public Board(int x, int y) {
		BOARD = new Vector();
		dimension_x = x;
		dimension_y = y;
		for (int contx=0;contx<x;contx++){
			BOARD.add(contx,new Vector());
			for (int conty=0;conty<y;conty++){
				((Vector)BOARD.elementAt(contx)).add(conty,null);
				//System.out.println(contx + " " + conty);
			}
		}
	}
	*/
	
	// if tag is defined then full the TAB with null_piece
	public Board(int x, int y) {
		BOARD = new Vector();
		DIMENSION_X = x;
		DIMENSION_Y = y;
		for (int contx=0;contx<x;contx++){
			BOARD.add(contx,new Vector());
			for (int conty=0;conty<y;conty++){
				((Vector)BOARD.elementAt(contx)).add(conty,new NullPiece());
				//System.out.println(contx + " " + conty);
			}
		}

	}
	
	public Board backup(){
		Board BACKUP = new Board(DIMENSION_X,DIMENSION_Y);
		for (int contx=0;contx<DIMENSION_X;contx++){
			for (int conty=0;conty<DIMENSION_Y;conty++){
				BACKUP.removePiece(contx,conty);
				BACKUP.addPiece(contx,conty,this.getPiece(contx,conty));
			}
		}
		return BACKUP;
	}
	
	public void print (){
		print(10);
	}
	
	public void print (int total_space){
		print(total_space," ", " ");
	}
	
	public void print (int total_space, String internal_space, String external_space){
		String space  = "---------------------";
		for (int conty=0;conty<DIMENSION_Y;conty++){
			System.out.print(space.subSequence(0,total_space+1));
		}
		System.out.println('-');
		for (int contx=0;contx<DIMENSION_X;contx++){
			System.out.print('|');
			for (int conty=0;conty<DIMENSION_Y;conty++){
				String temp = "";
				if (((Vector)BOARD.elementAt(contx)).elementAt(conty) == null){
					temp = "";
				}else{
					temp = ((Piece)((Vector)BOARD.elementAt(contx)).elementAt(conty)).getString();
				}
				temp = internal_space + temp + "            ";
				System.out.print(temp.subSequence(0,total_space-external_space.length()));
				System.out.print(external_space+"|");
			}
			System.out.println();
			for (int conty=0;conty<DIMENSION_Y;conty++){
				System.out.print(space.subSequence(0,total_space+1));
			}
			System.out.println('-');
		}
	}
	
	public String board2string (){
		String result = "";
		for (int contx=0;contx<DIMENSION_X;contx++){
			for (int conty=0;conty<DIMENSION_Y;conty++){
				String temp = " ";
				if (((Vector)BOARD.elementAt(contx)).elementAt(conty) == null){
					temp = " ";
				}else{
					temp = ((Piece)((Vector)BOARD.elementAt(contx)).elementAt(conty)).getString();
				}
				if (temp.equals("")){
					temp = " ";
				}
				result += "<" + temp + ">";
			}
		}
		return result;
	}
	
	
	public Piece getPiece(int x, int y){
		if (validCell(x,y)){
			return (Piece)((Vector)BOARD.elementAt(x)).elementAt(y);
		}
		return (Piece)(new NullPiece());
	}
	
	public boolean addPiece(int x, int y, Piece element){
		if (validCell(x,y)){
			((Vector)BOARD.elementAt(x)).remove(y);
			((Vector)BOARD.elementAt(x)).add(y, element);
			return true;
		}
		return false;
	}

	public boolean replacePiece(int x, int y, Piece element){
		if (validCell(x,y)){
			((Vector)BOARD.elementAt(x)).remove(y);
			((Vector)BOARD.elementAt(x)).add(y, element);
			return true;
		}
		return false;
	}
	
	public boolean replacePiece(int x, int y,int x2, int y2){
		if (validCell(x,y)){
			Piece temp = (Piece)((Vector)BOARD.elementAt(x)).get(y);
			Piece temp2 = (Piece)((Vector)BOARD.elementAt(x2)).get(y2);
			//System.out.print(temp.getClass() + " => " + ((Vector)BOARD.elementAt(x2)).get(y2).getClass());
			((Vector)BOARD.elementAt(x)).remove(y);
			//((Vector)BOARD.elementAt(x)).add(y, (piece)((Vector)BOARD.elementAt(x2)).get(y2));
			((Vector)BOARD.elementAt(x)).add(y, temp2);
			((Vector)BOARD.elementAt(x2)).remove(y2);
			((Vector)BOARD.elementAt(x2)).add(y2, temp);
			
			return true;
		}
		return false;
	}
	
	
	public boolean removePiece(int x, int y){
		if (validCell(x,y)){
			((Vector)BOARD.elementAt(x)).remove(y);
			((Vector)BOARD.elementAt(x)).add(y, new NullPiece());
			return true;
		}
		return false;
	}
	
	public boolean deletePiece(int x, int y){
		return removePiece(x,y);
	}
	
	public boolean validCell(int x, int y){
		if ((x >= 0) && (y >= 0)){
			if ((x < DIMENSION_X) && (y < DIMENSION_Y)){
				return true;
			}
		}
		return false;
	}
	
	public boolean movePiece(int x1, int y1, int x2, int y2){
		if (validCell(x1,y1) && validCell(x2,y2)){
		    Piece temp = (Piece)((Vector)BOARD.elementAt(x1)).elementAt(y1);
			((Vector)BOARD.elementAt(x2)).remove(y2);
			((Vector)BOARD.elementAt(x2)).add(y2, temp);
			((Vector)BOARD.elementAt(x1)).remove(y1);
			((Vector)BOARD.elementAt(x1)).add(y1, new NullPiece());
			return true;
		}
		return false;
	}
	
	public int getLines(){
		return DIMENSION_X;
	}

	public int getColumns(){
		return DIMENSION_Y;
	}
	
	public int contPieces(Class c1){
		int cont = 0;
		for (int contx=0;contx<DIMENSION_X;contx++){
			for (int conty=0;conty<DIMENSION_Y;conty++){
				Piece P = (Piece)((Vector)BOARD.elementAt(contx)).elementAt(conty);
				if (P != null){
				  if (P.getClass() == c1){
					  cont++;
				  }
				}
			}
		}
	    return cont;
	}
	
	public int contPieces(int Player){
		int cont = 0;
		for (int contx=0;contx<DIMENSION_X;contx++){
			for (int conty=0;conty<DIMENSION_Y;conty++){
				Piece P = (Piece)((Vector)BOARD.elementAt(contx)).elementAt(conty);
				if (P != null){
				  if (P.getPlayer() == Player ){
					  cont++;
				  }
				}
			}
		}
	    return cont;
	}
}
