package manager;


/**
 * This class extends the generic board game class {@link Game} to support a chess game.
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * 
 * @see   Game
 */

public class ChessGame extends Game {
	private int PLAYER1 = -1;
	private int PLAYER2 = -1;
	public boolean CHECK = false;
	public boolean WAITING_FOR_CHOICE = false;
	public Action LAST_ACTION = new Action(Action.NULL_ACTION);
	
	public boolean LEFT_ROOK_PLAYER1_MOVED = false;
	public boolean RIGHT_ROOK_PLAYER1_MOVED = false;
	public boolean KING_PLAYER1_MOVED = false;
	
	public boolean LEFT_ROOK_PLAYER2_MOVED = false;
	public boolean RIGHT_ROOK_PLAYER2_MOVED = false;
	public boolean KING_PLAYER2_MOVED = false;
	
	public ChessGame(String name){
		super(name);
		GAME_TYPE = "chess";
		MAX_PLAYERS = 2;
		MIN_PLAYERS = 2;
	}
	
	public ChessGame(String name, int player1) {
		super(name, player1);
		GAME_TYPE = "chess";
		MAX_PLAYERS = 2;
		MIN_PLAYERS = 2;
		PLAYER1 = player1;
	}

	public void startGame(){
		System.out.println("# of Players:" + PLAYERS.size());
		if (PLAYERS.size() == 2){
			PLAYER1 = ((Integer)PLAYERS.get(0)).intValue();
			PLAYER2 = ((Integer)PLAYERS.get(1)).intValue();
			ACTUAL_PLAYER = PLAYER1;
			WAITING_PLAYERS=false;
			GAME_ENDED = false;
			insertPieces();
		}else{
			System.err.println("Wrong number of players:" + PLAYERS.size());
		}
	}
	
	public boolean inCheck(int Player){
		Position KingPosition = findKing(Player);
		int KingX = KingPosition.X;
		int KingY = KingPosition.Y;
		for (int x=0;x<=8;x++){
			for (int y=0;y<=8;y++){
				if (BOARD.getPiece(x,y) instanceof ChessPiece){
					if (((ChessPiece)BOARD.getPiece(x,y)).PLAYER != Player){

						ChessPiece P1 = (ChessPiece)BOARD.getPiece(x,y);
						int x2 = KingX;
						int y2 = KingY;
						int x1 = x;
						int y1 = y;
						
						if (P1.get_type().equalsIgnoreCase(ChessPiece.PAWN)){
							if (Player == PLAYER1){
								if((x2-x1==-1) && Math.abs(y2-y1)==1){
									return true;
								}
							}else if (Player == PLAYER2){
								if((x2-x1==1) && Math.abs(y2-y1)==1){
									return true;
								}
							}	
						}else if (P1.get_type().equalsIgnoreCase(ChessPiece.KNIGHT)){
							if (((Math.abs(x2-x1)==1) && (Math.abs(y2-y1)==2)) || ((Math.abs(x2-x1)==2) && (Math.abs(y2-y1)==1))){
								return true;
							}
						}else if (P1.get_type().equalsIgnoreCase(ChessPiece.BISHOP)){
							if (Math.abs(x2-x1)== Math.abs(y2-y1) && y2 != y1){
								if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
									return true;
								}
							}
						}else if (P1.get_type().equalsIgnoreCase(ChessPiece.ROOK)){
							if ((x2-x1 == 0 && Math.abs(y2-y1)>0) || (y2-y1 == 0 && Math.abs(x2-x1)>0)){
								if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
									return true;
								}
							}
						}else if (P1.get_type().equalsIgnoreCase(ChessPiece.QUEEN)){
							if ((x2-x1 == 0 && Math.abs(y2-y1)>0) || (y2-y1 == 0 && Math.abs(x2-x1)>0)){
								if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
									return true;
								}
							}else if (Math.abs(x2-x1)== Math.abs(y2-y1) && y2 != y1){
								if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
									return true;
								}
							}
						}else if (P1.get_type().equalsIgnoreCase(ChessPiece.KING)){
							if ((Math.abs(y2-y1)<=1) && (Math.abs(x2-x1)<=1) && (Math.abs(y2-y1) + Math.abs(x2-x1) == 1)){
									return true;
							}
						}

						
						
					}
				}
			}
		}
		return false;
	}
	
	public Position findKing(int Player){
		for (int x=0;x<=8;x++){
			for (int y=0;y<=8;y++){
				if (BOARD.getPiece(x,y).getClass() != NullPiece.class){
					if ((((ChessPiece)BOARD.getPiece(x,y)).get_type().equalsIgnoreCase(ChessPiece.KING)) && ((ChessPiece)BOARD.getPiece(x,y)).PLAYER == Player){
						return new Position(x,y);
					}
				}
			}
		}
		return new Position(-1,-1);
	}
	
	public boolean move(int x1, int y1, int x2, int y2, int Player){
		boolean eat = false;
		Piece eaten_piece = BOARD.getPiece(x2,y2);
		boolean CheckMate = false;
		if (canMove(x1, y1, x2, y2, Player)){
			int x3 = x2;
			int y3 = y2;
			if ((y1 != y2) && eaten_piece.getClass() == NullPiece.class && ((ChessPiece)BOARD.getPiece(x1,y1)).get_type().equalsIgnoreCase(ChessPiece.PAWN)){
				eaten_piece = BOARD.getPiece(x1,y2);
				BOARD.removePiece(x1,y2);
				//System.out.println(x1 + " => " + y2);
				x3 = x1;
				y3 = y2;
			}
			if (eaten_piece.getClass() != NullPiece.class){
				eat = true;
			}
			
			//System.err.println(x1 + " " + y1 + " => "+ ((Chess_piece)BOARD.getPiece(x1,y1)).get_type());
			if ((x1==0 && y1 == 0) && (!LEFT_ROOK_PLAYER2_MOVED)){
				LEFT_ROOK_PLAYER2_MOVED = true;
			}else if ((x1==0 && y1 == 7) && (!RIGHT_ROOK_PLAYER2_MOVED)){
				RIGHT_ROOK_PLAYER2_MOVED = true;
			}else if ((x1==7 && y1 == 0) && (!LEFT_ROOK_PLAYER1_MOVED)){
				LEFT_ROOK_PLAYER1_MOVED = true;
			}else if ((x1==7 && y1 == 7) && (!RIGHT_ROOK_PLAYER1_MOVED)){
				RIGHT_ROOK_PLAYER1_MOVED = true;
			}
			
			if (((ChessPiece)BOARD.getPiece(x1,y1)).get_type().equalsIgnoreCase(ChessPiece.KING)){
				if (Player == PLAYER1){
					KING_PLAYER1_MOVED = true;
				}else{
					KING_PLAYER2_MOVED = true;
				}
			}
			
			boolean change_pawn = false;
			if (((ChessPiece)BOARD.getPiece(x1,y1)).get_type().equalsIgnoreCase(ChessPiece.PAWN)){
				if (x2==0 || x2==7){
					change_pawn = true;
				}
			}
			
			// Castling [roque]
			if (x1==7 && y1 == 4 && ((ChessPiece)BOARD.getPiece(x1,y1)).get_type().equalsIgnoreCase(ChessPiece.KING) && Math.abs(y2-y1)>1){
				if (y2==2){
					BOARD.movePiece(7,0,7,3);
				}else{
					BOARD.movePiece(7,7,7,5);
				}
			}else if (x1==0 && y1 == 4 && ((ChessPiece)BOARD.getPiece(x1,y1)).get_type().equalsIgnoreCase(ChessPiece.KING) && Math.abs(y2-y1)>1){
				if (y2==2){
					BOARD.movePiece(0,0,0,3);
				}else{
					BOARD.movePiece(0,7,0,5);
				}
			}

			BOARD.movePiece(x1,y1,x2,y2);
			
			if (change_pawn){
				if (x2==0){
					BOARD.addPiece(x2, y2,new ChessPiece("X",PLAYER1,ChessPiece.QUEEN));
				}else{
					BOARD.addPiece(x2, y2,new ChessPiece("O",PLAYER2,ChessPiece.QUEEN));
				}
			}
			
			//System.err.println(x2 + " " + y2 + " => "+ ((Chess_piece)BOARD.getPiece(x2,y2)).get_type());
			//if ((Player == PLAYER1) && (x2==7)){
			//	((checkers_piece)BOARD.getPiece(x2,y2)).TYPE = checkers_piece.CHECKER;
			//}else if((Player == PLAYER2) && (x2==0)){
			//	((checkers_piece)BOARD.getPiece(x2,y2)).TYPE = checkers_piece.CHECKER;
			//}
			// TODO
			
			String CheckTag = "";
			if (Player == PLAYER1 && inCheck(PLAYER2)){
				CheckTag = "<CHECK MESSAGE>PLAYER2 IN CHECK</CHECK MESSAGE>";
			}else if (Player == PLAYER2 && inCheck(PLAYER1)){
				CheckTag = "<CHECK MESSAGE>PLAYER1 IN CHECK</CHECK MESSAGE>";
			}
			
			//System.out.println("InCheck: " + PLAYER1 + " " + inCheck(PLAYER1) + "; InCheck: " + PLAYER2 + " " +inCheck(PLAYER2));
			if (eat){
				LAST_ACTION = new MoveAndEatAction(x1,y1,x2,y2,BOARD.getPiece(x2,y2),eaten_piece,x3,y3);
			}else{
				LAST_ACTION = new MoveAction(x1,y1,x2,y2,BOARD.getPiece(x2,y2));
			}
			if (ACTUAL_PLAYER == PLAYER1){
				ACTUAL_PLAYER = PLAYER2;
			}else{
				ACTUAL_PLAYER = PLAYER1;
			}
			
			//System.out.println("Original:" + BOARD.board2string());
			//System.out.println("Backup  :" + BACKUP_BOARD.board2string());
			
			CheckMate = CheckMate(ACTUAL_PLAYER);
			if (CheckMate){
				if (ACTUAL_PLAYER==PLAYER1){
					CheckTag = "<CHECK MESSAGE>PLAYER1 IN CHECKMATE</CHECK MESSAGE>";
					WINNER = PLAYER2;
				}else{
					CheckTag = "<CHECK MESSAGE>PLAYER2 IN CHECKMATE</CHECK MESSAGE>";
					WINNER = PLAYER1;
				}
				GAME_ENDED = true;
			}
			LAST_MESSAGE = "<LAST ACTION>"+LAST_ACTION.action2String()+"</LAST ACTION>"+CheckTag;
			if (CheckMate){
				System.out.println("   CheckMate: " + CheckMate);
				//System.out.println("   "+ LAST_MESSAGE);
			}
			return true;
		}
		return false;
	}
	
	
	
	private boolean CheckMate(int Player) {
		Position p = findKing(Player);
		if (!(inCheck(Player))){
			return false;
		}
		int KingX = p.X;
		int KingY = p.Y;
		if (canMoveAnywhere(KingX,KingY,ChessPiece.KING)){
			return false;
		}
		
		for (int cx=0;cx<=8;cx++){
			for (int cy=0;cy<=8;cy++){
				if (BOARD.getPiece(cx,cy) instanceof ChessPiece){
					ChessPiece P1 = (ChessPiece)BOARD.getPiece(cx,cy);
					if ((P1.PLAYER == Player && !P1.get_type().equalsIgnoreCase(ChessPiece.KING))){
						if (canMoveAnywhere(cx,cy,P1.get_type())){
							return false;
						}
					}
				}
			}
		}
			
			
		return true;
	}

	private boolean canMoveAnywhere(int x, int y, String pieceType) {
		System.out.println("canMoveAnywhere: (" + x +","+y + ")" + pieceType );
		if (pieceType.equalsIgnoreCase(ChessPiece.KING)){
			for (int cx=-1;cx<=1;cx++){
				for (int cy=-1;cy<=1;cy++){
					if (cx != 0 || cy != 0){
						if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
							System.out.println("   King can move to ("+(x+cx)+","+(y+cy)+")");
							return true;
						}
					}
				}
			}
		}else if (pieceType.equalsIgnoreCase(ChessPiece.PAWN)){
			for (int cx=-2;cx<=2;cx++){
				for (int cy=-1;cy<=1;cy++){
					if (cx != 0 || cy != 0){
						if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
							System.out.println("   Pawn can move to ("+(x+cx)+","+(y+cy)+")");
							return true;
						}
					}
				}
			}
		}else if (pieceType.equalsIgnoreCase(ChessPiece.KNIGHT)){
			for (int cx=-2;cx<=2;cx++){
				for (int cy=-2;cy<=2;cy++){
					if (Math.abs(cx)+Math.abs(cy) == 3){
						if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
							System.out.println("   Knight can move to ("+(x+cx)+","+(y+cy)+")");
							return true;
						}
					}
				}
			}
		}else if (pieceType.equalsIgnoreCase(ChessPiece.ROOK)){
			for (int cx=-8;cx<=8;cx++){
				for (int cy=-8;cy<=8;cy++){
					if ((cx==0 || cy==0) && !(cx==0 && cy==0)){
						if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
							System.out.println("   Rook can move to ("+(x+cx)+","+(y+cy)+")");
							return true;
						}
					}
				}
			}
		}else if (pieceType.equalsIgnoreCase(ChessPiece.BISHOP)){
			for (int cx=-8;cx<=8;cx++){
				if (canMove(x,y,x+cx,y+cx,ACTUAL_PLAYER)){
					System.out.println("   Bishop can move to ("+(x+cx)+","+(y+cx)+")");
					return true;
				}
				int cy = -cx;
				if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
					System.out.println("   Bishop can move to ("+(x+cx)+","+(y+cy)+")");
					return true;
				}
			}
		}else if (pieceType.equalsIgnoreCase(ChessPiece.QUEEN)){
			for (int cx=-8;cx<=8;cx++){
				for (int cy=-8;cy<=8;cy++){
					if ((cx==0 || cy==0) && !(cx==0 && cy==0)){
						if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
							System.out.println("   Queen can move to ("+(x+cx)+","+(y+cy)+")");
							return true;
						}
					}
					if (cx!=0 && Math.abs(cx)==Math.abs(cy)){
						if (canMove(x,y,x+cx,y+cy,ACTUAL_PLAYER)){
							System.out.println("   Queen can move to ("+(x+cx)+","+(y+cy)+")");
							return true;
						}	
					}
				}
			}
		}
		
		
		
		return false;
	}

	public boolean canMove(int x1, int y1, int x2, int y2, int Player){	
		if (canMoveAux(x1,y1,x2,y2,Player)){
			boolean eat = false;
			Piece eaten_piece = BOARD.getPiece(x2,y2);
			Board BOARD_BACKUP = BOARD.backup();
			if (canMoveAux(x1, y1, x2, y2, Player)){
				int x3 = x2;
				int y3 = y2;
				if ((y1 != y2) && eaten_piece.getClass() == NullPiece.class && ((ChessPiece)BOARD.getPiece(x1,y1)).get_type().equalsIgnoreCase(ChessPiece.PAWN)){
					eaten_piece = BOARD.getPiece(x1,y2);
					BOARD.removePiece(x1,y2);
					//System.out.println(x1 + " => " + y2);
					x3 = x1;
					y3 = y2;
				}
				if (eaten_piece.getClass() != NullPiece.class){
					eat = true;
				}
				BOARD.movePiece(x1,y1,x2,y2);
				
				if (inCheck(Player)){
					BOARD = BOARD_BACKUP;
					return false;			
				}else{
					BOARD = BOARD_BACKUP;
					return true;
				}	
			}
			BOARD = BOARD_BACKUP;
		}
		return false;
		
		//return canMoveAux(x1,y1,x2,y2,Player);
	}
	
	
	public boolean canMoveAux(int x1, int y1, int x2, int y2, int Player){
		//System.out.println("Trying to move chess.");
		if (GAME_ENDED || WAITING_PLAYERS){
			return false;
		}
		if (Player != ACTUAL_PLAYER){
			return false;
		}
		
		if (!BOARD.validCell(x1,y1) || !BOARD.validCell(x2,y2)){
			return false;
		}
		
		if (BOARD.getPiece(x1,y1).getClass() == NullPiece.class){
			return false;
		}
		
		boolean null_destination = true;
		if (BOARD.getPiece(x2,y2).getClass() != NullPiece.class){
			null_destination = false;
			if (((ChessPiece)BOARD.getPiece(x2,y2)).PLAYER == Player){
				return false;	
			}
		}

		if (BOARD.getPiece(x1,y1).getClass() == NullPiece.class){
			return false;
		}
		
		ChessPiece P1 = (ChessPiece)BOARD.getPiece(x1,y1); 
		if (P1.getPlayer() != Player){
			System.out.println("player:" + P1.getPlayer());
			return false;
		}
		
		// TODO
		/////////////// VERIFYING NORMAL MOVEMENT (NO PIECE EATEN)
		
		//System.out.println("Moving chess: player " + Player + " x " + ACTUAL_PLAYER+ "; piece "+ P1.get_type());
		if (P1.get_type().equalsIgnoreCase(ChessPiece.PAWN)){
			if (Player == PLAYER1){
				if ((x2-x1==-1) && (y2==y1) && (null_destination)){
					return true;
				}else if ((x2-x1==-2) && (y2==y1) && (x1==6) && (null_destination)){
					return true;
				}else if((x2-x1==-1) && Math.abs(y2-y1)==1){
					if (!(null_destination)){
						return true;
					}else if (BOARD.getPiece(x1,y2).getClass() != NullPiece.class){
						if (((ChessPiece)BOARD.getPiece(x1,y2)).PLAYER != Player && ((ChessPiece)BOARD.getPiece(x1,y2)).get_type().equalsIgnoreCase(ChessPiece.PAWN)){ 
							if ((LAST_ACTION.getClass() == MoveAction.class)){
								if (((MoveAction)LAST_ACTION).TO_X == x1 && ((MoveAction)LAST_ACTION).TO_Y == y2 && ((ChessPiece)((MoveAction)LAST_ACTION).ACTOR).get_type().equalsIgnoreCase(ChessPiece.PAWN) && ((MoveAction)LAST_ACTION).FROM_X == x1-2){
									return true;
								}
							}	  
						}
					}					
				}
			}else if (Player == PLAYER2){
				if ((x2-x1==1) && (y2==y1) && (null_destination)){
					return true;
				}else if ((x2-x1==2) && (y2==y1) && (x1==1) && (null_destination)){
					return true;
				}else if((x2-x1==1) && Math.abs(y2-y1)==1){
					if (!(null_destination)){
						return true;
					}else if (BOARD.getPiece(x1,y2).getClass() != NullPiece.class){
						if (((ChessPiece)BOARD.getPiece(x1,y2)).PLAYER != Player && ((ChessPiece)BOARD.getPiece(x1,y2)).get_type().equalsIgnoreCase(ChessPiece.PAWN)){ 
							if ((LAST_ACTION.getClass() == MoveAction.class)){
								if (((MoveAction)LAST_ACTION).TO_X == x1 && ((MoveAction)LAST_ACTION).TO_Y == y2 && ((ChessPiece)((MoveAction)LAST_ACTION).ACTOR).get_type().equalsIgnoreCase(ChessPiece.PAWN)  && ((MoveAction)LAST_ACTION).FROM_X == x1+2){
									return true;   
								}
							}	  
						}
					}
					
				}

			}
			
		}else if (P1.get_type().equalsIgnoreCase(ChessPiece.KNIGHT)){
			if (((Math.abs(x2-x1)==1) && (Math.abs(y2-y1)==2)) || ((Math.abs(x2-x1)==2) && (Math.abs(y2-y1)==1))){
				return true;
			}
		}else if (P1.get_type().equalsIgnoreCase(ChessPiece.BISHOP)){
			if (Math.abs(x2-x1)== Math.abs(y2-y1) && y2 != y1){
				if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
					return true;
				}
			}
		}else if (P1.get_type().equalsIgnoreCase(ChessPiece.ROOK)){
			if ((x2-x1 == 0 && Math.abs(y2-y1)>0) || (y2-y1 == 0 && Math.abs(x2-x1)>0)){
				if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
					return true;
				}
			}
		}else if (P1.get_type().equalsIgnoreCase(ChessPiece.QUEEN)){
			if ((x2-x1 == 0 && Math.abs(y2-y1)>0) || (y2-y1 == 0 && Math.abs(x2-x1)>0)){
				if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
					return true;
				}
			}else if (Math.abs(x2-x1)== Math.abs(y2-y1) && y2 != y1){
				if (!thereIsSomeoneInTheWay(x1,y1,x2,y2)){
					return true;
				}
			}
		}else if (P1.get_type().equalsIgnoreCase(ChessPiece.KING)){
			if ((Math.abs(y2-y1)<=1) && (Math.abs(x2-x1)<=1) && (Math.abs(y2-y1) + Math.abs(x2-x1) == 1)){
					return true;
			}else{
				if (Player == PLAYER1){
					if (KING_PLAYER1_MOVED == false){
						// Kingside Castling [ROQUE A DIREITA PLAYER1]
						if (x1==7 && y1==4 && x2 == 7 && y2 == 6){
							if (RIGHT_ROOK_PLAYER1_MOVED == false && BOARD.getPiece(7,5).getClass()==NullPiece.class && BOARD.getPiece(7,6).getClass()==NullPiece.class){
								if (BOARD.getPiece(x1,7) instanceof ChessPiece){
									if (((ChessPiece)BOARD.getPiece(x1,7)).PLAYER == Player && ((ChessPiece)BOARD.getPiece(x1,7)).get_type().equalsIgnoreCase(ChessPiece.ROOK)){ 
										Board BACK_BOARD = BOARD.backup();
										if (canMove(x1,y1,x2,5,Player)){
											BOARD.movePiece(x1,y1,x2,y2);
											BOARD.movePiece(x1,y1,x2,5);
											boolean CanCastling = !(inCheck(Player));
											BOARD = BACK_BOARD;
											System.out.println("Can roque!");
											return CanCastling;
										}
									}
								}
							}
						}else if (x1==7 && y1==4 && x2 == 7 && y2 == 2){
							// Queenside Castling [ROQUE A ESQUERDA PLAYER1]
								if (LEFT_ROOK_PLAYER1_MOVED == false && BOARD.getPiece(7,1).getClass()==NullPiece.class && BOARD.getPiece(7,2).getClass()==NullPiece.class && BOARD.getPiece(7,3).getClass()==NullPiece.class){
									if (BOARD.getPiece(x1,0) instanceof ChessPiece){
										if (((ChessPiece)BOARD.getPiece(x1,0)).PLAYER == Player && ((ChessPiece)BOARD.getPiece(x1,7)).get_type().equalsIgnoreCase(ChessPiece.ROOK)){ 
											Board BACK_BOARD = BOARD.backup();
											if (canMove(x1,y1,x2,3,Player)){
												BOARD.movePiece(x1,y1,x2,y2);
												BOARD.movePiece(x1,y1,x2,3);
												boolean CanCastling = !(inCheck(Player));
												BOARD = BACK_BOARD;
												System.out.println("Can roque!");
												return CanCastling;
											}
										}
									}
								}
						}
					}
				}
				//TODO Castling [roque] player 2
				if (Player == PLAYER2){
					if (KING_PLAYER2_MOVED == false){
						// Kingside Castling [ROQUE A DIREITA PLAYER1]
						if (x1==0 && y1==4 && x2 == 0 && y2 == 6){
							if (RIGHT_ROOK_PLAYER2_MOVED == false && BOARD.getPiece(0,5).getClass()==NullPiece.class && BOARD.getPiece(0,6).getClass()==NullPiece.class){
								if (BOARD.getPiece(x1,0) instanceof ChessPiece){
									if (((ChessPiece)BOARD.getPiece(x1,0)).PLAYER == Player && ((ChessPiece)BOARD.getPiece(x1,0)).get_type().equalsIgnoreCase(ChessPiece.ROOK)){ 
										Board BACK_BOARD = BOARD.backup();
										if (canMove(x1,y1,x2,5,Player)){
											BOARD.movePiece(x1,y1,x2,y2);
											BOARD.movePiece(x1,y1,x2,5);
											boolean CanCastling = !(inCheck(Player));
											BOARD = BACK_BOARD;
											System.out.println("Can roque!");
											return CanCastling;
										}
									}
								}
							}
						}else if (x1==0 && y1==4 && x2 == 0 && y2 == 2){
							// Queenside Castling [ROQUE A ESQUERDA PLAYER1]
								if (LEFT_ROOK_PLAYER1_MOVED == false && BOARD.getPiece(0,1).getClass()==NullPiece.class && BOARD.getPiece(0,2).getClass()==NullPiece.class && BOARD.getPiece(0,3).getClass()==NullPiece.class){
									if (BOARD.getPiece(x1,0) instanceof ChessPiece){
										if (((ChessPiece)BOARD.getPiece(x1,0)).PLAYER == Player && ((ChessPiece)BOARD.getPiece(x1,0)).get_type().equalsIgnoreCase(ChessPiece.ROOK)){ 
											Board BACK_BOARD = BOARD.backup();
											if (canMove(x1,y1,x2,3,Player)){
												BOARD.movePiece(x1,y1,x2,y2);
												BOARD.movePiece(x1,y1,x2,3);
												boolean CanCastling = !(inCheck(Player));
												BOARD = BACK_BOARD;
												System.out.println("Can roque!");
												return CanCastling;
											}
										}
									}
								}
							}
						}
					}
			}
		}
	
		return false;
	}

	
	public boolean isAdversary(Piece P1, int Player){
		int temp_player = P1.getPlayer();
		if ((temp_player == PLAYER1) || (temp_player == PLAYER2)){
			if (((Player == PLAYER1) || (Player == PLAYER2)) && temp_player != Player){
				return true;
			}
		}
		return false;
	}

	public boolean eat(int x, int y){
		if (BOARD.getPiece(x,y).getClass() == ChessPiece.class){
			if (BOARD.removePiece(x,y)){
				if (gameEnded()){
					System.out.println("End game. Winner: " + WINNER);
				}
				return true;
			}
		}
		return false;
	}

	public boolean canEat(int x, int y, int x2, int y2, int Player){
		if (BOARD.validCell(x2,y2)){
		  if (BOARD.getPiece(x2,y2).getClass() == ChessPiece.class){
			  return canMove(x,y,x2,y2,Player);
		  }
		}
		return true;
	}

	
	public boolean canAdd(int x, int y, Piece P){
		return false;
	}
	
	public boolean add(int x, int y, String Player){
		return false;
	}
	
	public void initializeBoard(){
		BOARD = new Board(8,8);
	}

	public void insertPieces(){
		for (int cont=0;cont<8;cont++){
			BOARD.addPiece(1, cont,new ChessPiece("O",PLAYER2,ChessPiece.PAWN));		
			BOARD.addPiece(6, cont,new ChessPiece("X",PLAYER1,ChessPiece.PAWN));
		}
		for (int cont=0;cont<2;cont++){
			BOARD.addPiece(0, Math.abs(7*cont),new ChessPiece("O",PLAYER2,ChessPiece.ROOK));
			BOARD.addPiece(7, Math.abs(7*cont),new ChessPiece("X",PLAYER1,ChessPiece.ROOK));
			
			BOARD.addPiece(0, Math.abs(7*cont-1),new ChessPiece("O",PLAYER2,ChessPiece.KNIGHT));
			BOARD.addPiece(7, Math.abs(7*cont-1),new ChessPiece("X",PLAYER1,ChessPiece.KNIGHT));
			
			BOARD.addPiece(0, Math.abs(7*cont-2),new ChessPiece("O",PLAYER2,ChessPiece.BISHOP));
			BOARD.addPiece(7, Math.abs(7*cont-2),new ChessPiece("X",PLAYER1,ChessPiece.BISHOP));	
		}
		
		BOARD.addPiece(0, 3,new ChessPiece("O",PLAYER2,ChessPiece.QUEEN));
		BOARD.addPiece(0, 4,new ChessPiece("O",PLAYER2,ChessPiece.KING));

		BOARD.addPiece(7, 3,new ChessPiece("X",PLAYER1,ChessPiece.QUEEN));
		BOARD.addPiece(7, 4,new ChessPiece("X",PLAYER1,ChessPiece.KING));
	}
	
	public boolean gameEnded(){
		if (GAME_ENDED){
			return true;
		}
		if (player1Won()){
			WINNER = PLAYER1;
			GAME_ENDED = true;
			return true;
		}else if (player2Won()){
			WINNER = PLAYER2;
			GAME_ENDED = true;
			return true;
		}
		return false;
	}
	
	public boolean player1Won(){
		if (WINNER != -1 && WINNER == PLAYER1){
			return true;
		}else if (won(PLAYER1)){
			WINNER = PLAYER1;
			return true;
		}
		return false;
	}
	public boolean player2Won(){
		if (WINNER != -1 && WINNER == PLAYER2){
			return true;
		}else if (won(PLAYER2)){
			WINNER = PLAYER2;
			return true;
		}
		return false;
	}	
	
	 
	public boolean won(int Player){
		// TODO
		return false;
	}
	
	
	public boolean thereIsSomeoneInTheWay(int x1, int y1,int x2, int y2){
		int add_x = 1;
		int add_y = 1;
		if (x2-x1 < 0){
			add_x = -1;
		}else if (x2==x1){
			add_x = 0;
		}
		if (y2-y1 < 0){
			add_y = -1;
		}else if (y2==y1){
			add_y = 0;
		}
		x1 += add_x;
		y1 += add_y;
		int cont = 0; // backup ;-)
		while (x1 != x2 || y1 != y2 && cont<10){
			cont++;
			//System.out.println("    |=> ("+x1+","+y1+")");
			if (BOARD.validCell(x1,y1)){
				if ((BOARD.getPiece(x1,y1).getClass() != NullPiece.class)){
					//System.out.println("   Peca aqui => ("+x1+","+y1+")");
					return true;
				}
			}
			x1 += add_x;
			y1 += add_y;
		}
		return false;
	}
	
}