package manager;

/**
 * This class implements any deflexion piece.
 * It extends a generic {@link Piece} to support piece orientation
 * (the direction in which the mirror is pointing to) and the type
 * of deflexion piece (pharoah, djed, pyramid or obelisk).
 * <p>
 * Deflexion pieces are used in the {@link DeflexionGame}
 *  
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * @see DeflexionGame
 * @see Piece
 */
public class DeflexionPiece extends Piece{
	final static String PHAROAH = "pharoah";
	final static String DJED = "djed";
	final static String PYRAMID = "pyramid";
	final static String OBELISK = "obelisk";
	
	private String orientation = "NE";
	private String type = PYRAMID;
	public DeflexionPiece(String id, int player) {
		super(id, player);
	}

	public DeflexionPiece(String id, int player, String type, String orientation) {
		super(id, player);
		this.orientation = orientation;
		this.type = type;
	}

	public DeflexionPiece(String id, int player, String type) {
		super(id, player);
		this.type = type;
	}
	
	public void update_orientation(String ori) {
		this.orientation = ori;
	}
	
	public String get_orientation(){
		return orientation;
	}
	
	public String get_type(){
		return type;
	}
	

	public String getString(){
		String tag_orientation = "??";
		if (orientation.equalsIgnoreCase("NE")){
			tag_orientation = "$\\";
		}else if (orientation.equalsIgnoreCase("NW")){
			tag_orientation = "/$";
		}else if (orientation.equalsIgnoreCase("SE")){
			tag_orientation = "$/";
		}else if (orientation.equalsIgnoreCase("SW")){
			tag_orientation = "\\$";
		}
		
		
		if (type.equalsIgnoreCase(PHAROAH)){
			return ID + "Pha" +ID;
		}else if (type.equalsIgnoreCase(OBELISK)){
			return ID + "Obe" + ID;
		}else if (type.equalsIgnoreCase(DJED)){
			return ID + "D"+tag_orientation + ID;
		}else if (type.equalsIgnoreCase(PYRAMID)){
			return ID + "P" +tag_orientation+ ID;
		}
		
		System.out.println(type + " " + tag_orientation);
		return "ERROR - invalid type: "+type;
	}
	
}
