package manager;

import java.util.ArrayList;
import java.util.List;


/**
 * This class is a generic class of Board Games.
 * The public interface is responsible for all allowed actions in any kind of board game and should be 
 * implemented in the specialized class.
 * Each game contains its {@link Board} and each Board contains its pieces (@link piece}
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * @see Board
 * @see Piece
 */
public abstract class Game {
	//protected String name = "Generic Game";
	protected String GAME_NAME = "Generic Game";
	protected int ACTUAL_PLAYER = -1;
	public Board BOARD = null;
	public int WINNER = -1;
	public boolean GAME_ENDED = false;
	public boolean DRAWN_GAME = false;
	public List PLAYERS = new ArrayList();
	public int CONT_ID = -1;  // stores the number of valid actions during a game
	public String GAME_TYPE = "generic game";
	public int MAX_PLAYERS = 2;
	public boolean WAITING_PLAYERS = true;
	public int MIN_PLAYERS = 0;
	public boolean GAME_STARTED = false;
	public String LAST_MESSAGE = " ";
	
	public Game(String name){
		GAME_NAME = name;
		initializeBoard();
		CONT_ID++;
		GAME_ENDED = false;
    }
	
	public Game(){
		GAME_NAME = "new game";
		PLAYERS = new ArrayList();
		initializeBoard();
		CONT_ID++;
		GAME_ENDED = false;
	}
	
	public int getActualPlayer(){
		return ACTUAL_PLAYER;
	}
	
	public Game(String name, int Player){
		GAME_NAME = name;
		PLAYERS.add(new Integer(Player));
		initializeBoard();
		CONT_ID++;
	}
	

	public void initializeBoard(){
		System.out.println("Trying to initialize board - game.");
		BOARD = null;
	}
	
	public boolean canAdd(int x, int y, int Player){
		return false;
	}
	
	public int getCurrentGameId(){
		return CONT_ID;
	}
	
	public GameStatus getStatus(){
		//return "<LOCAL ID:"+CONT_ID+"><GAME ENDED:"+GAME_ENDED+"><ACTUAL PLAYER:"+ACTUAL_PLAYER+">";
		return null;
	}
	
	public String currentBoard(){
		return BOARD.board2string();
	}
	
	public boolean move(int x1, int y1, int x2, int y2, int Player){
		System.out.println("Trying to move.");
		return false;
	}

	public boolean endTurn(int Player){
		System.out.println("Trying to end turn.");
		return false;
	}
	
	public boolean canMove(int x1, int y1, int x2, int y2, int Player){
		System.out.println("Trying to can move.");
		return false;
	}
	
	public boolean add(int x, int y, int Player){
		System.out.println("Trying to add.");
		return false;
	}
	
	public String getGameName(){
		return GAME_NAME;
	}

	public int getLines(){
		return BOARD.getLines();
	}
	
	public int getColumns(){
		return BOARD.getColumns();
	}
	
	public boolean gameEnded(){
		return GAME_ENDED;
	}
	
	public int getMaxPlayers(){
		return MAX_PLAYERS;
	}
	
	public int getMinPlayers(){
		return MIN_PLAYERS;
	}
	
	public String getGameType(){
		return GAME_TYPE;
	}

	public boolean rotate(int x, int y, String direction, int Player){
		return false;
	}
	
	public List getPlayers(){
		return PLAYERS;
	}
	
	public boolean joinGame(int playerID){
		if (WAITING_PLAYERS){
			if (PLAYERS.size() < MAX_PLAYERS){
				if (!(PLAYERS.contains(new Integer(playerID)))){
					PLAYERS.add(PLAYERS.size(),new Integer(playerID));
					System.out.println("Joining player: " + playerID);
					CONT_ID++;
					if (PLAYERS.size() == MAX_PLAYERS){
						startGame();
						ACTUAL_PLAYER = ((Integer)PLAYERS.get(0)).intValue();
						WAITING_PLAYERS = false;
					}
					return true;
				}
			}
		}
		return false;
	}
	
	protected void startGame(){
		WAITING_PLAYERS = false;
		System.out.println("generic class game => startGame");
	}
	
	public boolean waitingPlayers(){
		return WAITING_PLAYERS;
	}
	
	public String getMessage(){
		return LAST_MESSAGE;
	}	
	
	public boolean gameDraw(){
		return DRAWN_GAME;
	}	
	
}

