package netGameManager;


/**
 * This class contains the basic attributes for a message that is produced in the {@link GameServer}.
 * This message is the response for a {@link RequestMessage} for {@link GaneClient} field REQUEST came from some ({@link GameClient}) and the GameServer will try to answer the request.
 * The answer text is stored in the field RESPONSE and the field OK says if the request was or not attended.
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * 
 * @param OK (boolean) - status of the request (attended or not);
 * @param GENERAL_MESSAGE (String) - ;
 * @param SPECIFIC_MESSAGE (String) - ;
 * @param GAME_GENERAL_STATUS (String) - ;
 * @see   GameClient
 * @see   GameServer
 */


public class AnswerMessage {
	public boolean OK = false;
	public int GID = -1;
	public String GENERAL_MESSAGE = "";
	public String SPECIFIC_MESSAGE = "";
	public String GAME_GENERAL_STATUS = "";
	final static String PIECE_ADDED = "<ACTION PERFORMED>PIECE ADDED</ACTION PERFORMED>";
	final static String PIECE_MOVED = "<ACTION PERFORMED>PIECE MOVED</ACTION PERFORMED>";
	final static String PIECE_ROTATED = "<ACTION PERFORMED>PIECE ROTATED</ACTION PERFORMED>";
	final static String NEW_GAME = "<ACTION PERFORMED>NEW GAME CREATED</ACTION PERFORMED>";
	final static String JOIN_GAME = "<ACTION PERFORMED>PLAYER JOINED TO GAME</ACTION PERFORMED>";
	final static String CANNOT_JOIN_GAME = "<ACTION PERFORMED>CANNOT JOIN GAME</ACTION PERFORMED>";
	final static String CANNOT_ADD_PIECE = "<ACTION PERFORMED>CANNOT ADD PIECE</ACTION PERFORMED>";
	final static String CANNOT_MOVE_PIECE = "<ACTION PERFORMED>CANNOT MOVE PIECE</ACTION PERFORMED>";
	final static String CANNOT_ROTATE_PIECE = "<ACTION PERFORMED>CANNOT ROTATE PIECE</ACTION PERFORMED>";
	final static String DELETE_GAME = "<ACTION PERFORMED>GAME DELETED</ACTION PERFORMED>";
	final static String OUT_OF_PATTERN = "<ACTION PERFORMED>REQUEST OUT OF PATTERN</ACTION PERFORMED>";
	final static String INVALID_REQUEST_TYPE = "<ACTION PERFORMED>INVALID REQUEST TYPE</ACTION PERFORMED>";
	final static String SERVER_STATUS = "<ACTION PERFORMED>DESCRIBING SERVER STATUS</ACTION PERFORMED>";
	final static String GAME_STATUS = "<ACTION PERFORMED>DESCRIBING GAME STATUS</ACTION PERFORMED>";
	final static String GAME_BOARD = "<ACTION PERFORMED>SHOWING GAME BOARD</ACTION PERFORMED>";
	final static String GAME_LIST = "<ACTION PERFORMED>LISTING GAMES</ACTION PERFORMED>";
	final static String UNKNOWN_ERROR = "<ACTION PERFORMED>UNKNOWN ERROR</ACTION PERFORMED>";
	
	final static String GAME_DOES_NOT_EXIST = "<ACTION PERFORMED>GAME DOES NOT EXISTS</ACTION PERFORMED>";
	
    /*
     * GAME GENERAL STATUS VALUES
     */
	final static String GGS_GAME_DOES_NOT_EXIST = "<GGSTATUS>GAME DOES NOT EXISTS</GGSTATUS>";
	final static String GGS_NOTHING_TO_BE_DONE = "<GGSTATUS>NOTHING TO BE DONE</GGSTATUS>";
	final static String GGS_WAITING_FOR_PLAYERS = "<GGSTATUS>WAITING FOR PLAYERS</GGSTATUS>";
	final static String GGS_GAME_ENDED= "<GGSTATUS>GAME ENDED</GGSTATUS>";
	final static String GGS_YOUR_TURN = "<GGSTATUS>YOUR TURN</GGSTATUS>";
	final static String GGS_ADVERSARY_TURN = "<GGSTATUS>ADVERSARY'S TURN</GGSTATUS>";
	
	final static String DRAWN_GAME = "<DRAWNGAME>DRAWN GAME</DRAWNGAME>";
	
	final static String PLAYER1 = "<PLAYERNUMBER>1</PLAYERNUMBER>";
	final static String PLAYER2 = "<PLAYERNUMBER>2</PLAYERNUMBER>";
	
	public AnswerMessage(boolean status, String general, String specific, int gameID){
		OK = status;
		GENERAL_MESSAGE = general;
		SPECIFIC_MESSAGE = specific;
		GID = gameID;
	}
	
	public String message2string() {
		String answer_string = "<SERVER ANSWER>";
		if (OK){
			answer_string += "<STATUS>OK</STATUS>";
		}else{
			answer_string += "<STATUS>NOK</STATUS>";
		}
		answer_string += "<GENERAL ANSWER>" + GENERAL_MESSAGE+"</GENERAL ANSWER>";
		answer_string += "<SPECIFIC ANSWER>" + SPECIFIC_MESSAGE+"</SPECIFIC ANSWER>";
		answer_string += "<GAME STATUS>" + GAME_GENERAL_STATUS+"</GAME STATUS>";
		answer_string +="</SERVER ANSWER>";
		return answer_string;
	}
}

