package netGameManager;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class is responsible for all communication with {@link GameServer}.
 * GameServer manages instances of {@link manager.Game} games.
 * <p>This class has two main functionality: <p>1. create a new game in the server [using the parameters: type (String), name (String), Player (List of String)];
 * <p>2. join an an game in the server [using only the game_id (int)];
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * 
 * @see   GameServer
 * @see   manager.Game
 */

//@return - descreve o valor retornado por um mtodo
public class GameClient {
	protected static InetAddress ServerAddress = null;
	protected static int ServerPort = 7;
	protected static int GAME_ID=0;
	public String GAME_TYPE = "";
	protected String name = "Generic Game";
	protected String this_name = "";
	protected String Actual_player = "";
	public int lines = 0;
	public int columns = 0;
	//	public Board BOARD = null;
	public String Winner = "";
	public boolean GAME_ENDED = false;
	public List PLAYERS = null;
	public String GamePlayer = "";
		
	public GameClient(String name){
		this_name = name;
		//		initialize_board();
	}
	
/*	public GameClient(){
		this_name = "new game";
		PLAYERS = new Vector();
		initialize_board();
	}
*/
	/**
	 * Ask for the creation of a new game in a {@link GameServer}.
	 * @param game_type (String) - type of game to created (example: checkers, tic tac toe, deflexion);
	 * @param name (String) - label of the game (example: checker123);
	 * @param Players (List of String) - a list with the Players ID;
	 * @see GameServer
	 **/
	public GameClient(String game_type, String name, List Players){
		try { 
			ServerAddress = InetAddress.getByName("127.0.0.1");
		}
		catch (UnknownHostException uhe) {
		    System.err.println("No such host: 127.0.0.1");
		    System.exit(0);
		}
		this_name = name;
		PLAYERS = Players;
		GAME_TYPE = game_type;
		GamePlayer = Players.get(0).toString();
		String players_list = "";
		//if (Players.size()>0){
		//	players_list = "<";
		//}
		for (int cont=0;cont<Players.size();cont++){
			players_list = players_list + "<" +((String)Players.get(cont)) + ">";
		}
		String message = "<new game><"+game_type+"><"+name+">"+players_list;
		String server_response = send_message_to_server(message);
		System.out.println(server_response);
		
		Pattern kind_of_command = Pattern.compile("<NEW GAME:(.+)><LINES:(.+)><COLUMNS:(.+)><:>.+<:>",Pattern.CASE_INSENSITIVE);
//		Pattern kind_of_command = Pattern.compile("<NEW GAME"+tag+"(.+)>");
		Matcher m_class = kind_of_command.matcher(server_response);
		
		if (m_class.find()) {
			try{
			  GAME_ID = Integer.parseInt(m_class.group(1));
			  lines = Integer.parseInt(m_class.group(2));
			  columns = Integer.parseInt(m_class.group(3));
			}catch(Exception e){System.out.println("PARSER_PROBLEMS: " + server_response);}
			
		}else{
			System.out.println("New game presented problems: " + server_response);
		}
		
		initialize_board();
	}
	
	// JOIN_GAME
	/**
	 * @param game_id (int) - a ID of a game in the server, used to join a game;
	 */
	public GameClient(int game_id){
		try { 
			ServerAddress = InetAddress.getByName("127.0.0.1");
		}
		catch (UnknownHostException uhe) {
		    System.err.println("No such host: 127.0.0.1");
		    System.exit(0);
		}
		
		String message = message = "<join game><" + game_id +">";
		String server_response = send_message_to_server(message);
		System.out.println(server_response);
		
		//<JOIN GAME:0><NAME:game1><TYPE:checkers><LINES:8><COLUMNS:8><PLAYERS:j1;j2;><:><join game><0><:>OK
		Pattern kind_of_command = Pattern.compile("<JOIN GAME:(.+)><NAME:(.+)><TYPE:(.+)><LINES:(.+)><COLUMNS:(.+)><PLAYERS:(.+);(.+);><:><join game><.+><:>OK",Pattern.CASE_INSENSITIVE);
		Matcher m_class = kind_of_command.matcher(server_response);
		
		if (m_class.find()) {
			try{
			  GAME_ID = Integer.parseInt(m_class.group(1));
			  this_name = m_class.group(2);
			  GAME_TYPE = m_class.group(3);
			  lines = Integer.parseInt(m_class.group(4));
			  columns = Integer.parseInt(m_class.group(5));
			  PLAYERS = new ArrayList();
			  PLAYERS.add(0,m_class.group(6));
			  PLAYERS.add(1,m_class.group(7));
			  GamePlayer = m_class.group(7); 
			}catch(Exception e){System.out.println("PARSER_PROBLEMS: " + server_response);}	
		}else{
			System.out.println("Join game presented problems: " + server_response);
		}
	}

	
	protected static String send_message_to_server(String message){
		DatagramSocket ds = null;
		DatagramPacket rec, send;
		String line = "<ERROR>";
		rec = new DatagramPacket(new byte[65530], 65530);
		try {
			ds = new DatagramSocket();
			//while((line = br.readLine()) != null) {
			byte [] buf = message.getBytes();
			if (buf.length > 0) {
				send = new DatagramPacket(buf, buf.length, ServerAddress, ServerPort);
				rec.setLength(65530);
				ds.send(send);
				ds.receive(rec);
				line = new String(rec.getData(), 0, rec.getLength());
				System.out.println(line);
			}
		}
		catch (IOException ie) {
			System.err.println("IO exception in datagram test: " + ie);
		}
		finally {
			if (ds != null) try {
				ds.close();
				return line;
			} catch (Exception e) { }
		}
		return line;
	}
	
	/*public boolean can_add(int x1, int y1, piece P){
		System.out.println("Trying to verify can add.");
		return false;
	}*/
	
	public void initialize_board(){
		System.out.println("Trying to initialize board [client].");
//		BOARD = null;
	}
	
	public String current_board (){
		return send_message_to_server("<current board><"+GAME_ID+">");
	}
	
	public boolean can_add(int x, int y, String Player){
		return false;
	}
	
	/*
	public boolean add(int x, int y, piece P){
		System.out.println("Trying to add.");
		return false;
	}
	*/
	
	public boolean move(int x1, int y1, int x2, int y2, String Player){
		System.out.println("Trying to move.");
		return false;
	}

	public boolean end_turn(String Player){
		System.out.println("Trying to end turn.");
		return false;
	}
	
	public boolean can_move(int x1, int y1, int x2, int y2, String Player){
		System.out.println("Trying to can move.");
		return false;
	}
	
	public boolean add(int x, int y){
		System.out.println("Trying to add [player].");
		String message = "<add piece><"+GAME_ID+"><"+x+"><"+y+"><"+GamePlayer+">";
		String server_response = send_message_to_server(message);
		System.out.println(server_response);
		return false;
	}
	
	public String get_name(){
		return this_name;
	}

	public int get_lines(){
		return lines;
	}
	
	public int get_columns(){
		return columns;
	}
	
	public boolean end_game(){
		System.out.println("Trying to verify end game.");
		return false;
	}
	
}

