package netGameManager;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;


/*
 * COMUNICATION PATERN 
 *  ANSWER  PATERN:
 *   
 *  REQUEST PATERN: 
 *   LIST OF GAMES:	<REQUEST><RT>LIST GAMES</RT><GT>[all|checkers|deflexion|...]</GT><GS>[all|waiting_players]</GS></REQUEST>
 *   SERVER STATUS:	<REQUEST><RT>SERVER STATUS</RT></REQUEST>
 *   NEW GAME:		<REQUEST><RT>NEW GAME</RT><GT>[checkers|deflexion|...]CHECKERS</GT>[<GAME_NAME>[string game name]</GAME_NAME>][<PLAYER>[|player1_id]</PLAYER>]</REQUEST>
 *   GAME STATUS: 	<REQUEST><RT>GAME STATUS</RT><GID>[game_id]</GID></REQUEST>
 *   GAME BOARD: 	<REQUEST><RT>GAME BOARD</RT><GID>[game_id]</GID><PLAYER>[]</PLAYER></REQUEST>
 *   JOIN GAME:	   	<REQUEST><RT>JOIN GAME</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER></REQUEST>   
 *   DELETE GAME:	<REQUEST><RT>DELETE GAME</RT><GID>[game_id]</GID></REQUEST>
 *   ADD PIECE:    	<REQUEST><RT>ADD PIECE</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER><X>[x value]</X><Y>[y value]</Y></REQUEST>
 *   MOVE PIECE:    <REQUEST><RT>MOVE PIECE</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER><X1>[from x1]</X1><Y1>[from y1]</Y1><X2>[to x2]</X2><Y2>[to y2]</Y2></REQUEST>
 *   ROTATE PIECE:	<REQUEST><RT>ROTATE PIECE</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER><X>[x value]</X><Y>[y value]</Y><SENSE>[CLOCKWISE|ANTICLOCKWISE]</SENSE></REQUEST>
 *   
 *   
 * RT 	= REQUEST TYPE
 * GT 	= GAME TYPE : checkers, tictactoe, deflexion
 * GID 	= GAME ID
 * GS	= GAME STATUS
 * WT 	= WAITING PLAYERS
 * 
 * java NetGameManager/MyTest 127.0.0.1 7 "<new game><checkers><game1><j1><j2>"
 * java NetGameManager/MyTest 127.0.0.1 7 "<new game><tictactoe><game2><j1><j2>"
 * 
 * java NetGameManager/MyTest 127.0.0.1 7 "<join game><0>"
 * java NetGameManager/MyTest 127.0.0.1 7 "<add piece><game_id><x><y>"
*/

/**
 * This class is responsible for manage several intances of {@link manager.Game}.
 * It opens a TCP port to comunicate with players ({@link GameClient})
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * 
 * @param port (int) - set the port that will listen the players.It is an optional parameter, which default value is 7;
 * @param name (String) - label of the game (example: checker123);
 * @param PLAYER (List of String) - a list with the Players ID;
 * @see   GameClient
 * @see   manager.Game
 */
public class GameServer	{    
    protected Thread tcpThread;
    protected Thread udpThread;
    protected static Hashtable GAMES = new Hashtable();
    //protected static List GAMES = new ArrayList();
    protected static Hashtable GAMES_PLAYERS = new Hashtable();
    //protected static List GAMES_PLAYERS = new ArrayList();
    protected static Integer game_last_id = new Integer(-1);
  
    /**
     * Default port on which to provide TCP and UDP
     * echo services.
     * This is not final, so a caller could change it
     * to provide echo services on a different port.
     */
    public static int ECHO_PORT = 1027;

    /** 
     * Create a DualServer object to offer echo service
     * on TCP and UDP ports on the default IP address.
     */
    public GameServer() {
    	try {
    		tcpThread = new StreamEchoService(ECHO_PORT);
    	} catch (IOException ie) {
    		System.err.println("Couldn't start TCP echo service: " + ie);
    	}
    	try {
    		udpThread = new DatagramEchoService(ECHO_PORT);
    	} catch (IOException ie) {
    		System.err.println("Couldn't start UDP echo service: " + ie);
    	}
    }

    public GameServer(int port) {
    	ECHO_PORT = port;
    	try {
    		tcpThread = new StreamEchoService(ECHO_PORT);
    	} catch (IOException ie) {
    		System.err.println("Couldn't start TCP echo service: " + ie);
    	}
    	try {
    		udpThread = new DatagramEchoService(ECHO_PORT);
    	} catch (IOException ie) {
    		System.err.println("Couldn't start UDP echo service: " + ie);
    	}
    }
    
    
    /**
     * Start up this echo service.
     */
    public void startEchoService() {
	tcpThread.start();
	udpThread.start();
	return;
    }

    public static AnswerMessage executeRequest(RequestMessage req_message){  	
    	//RequestMessage req_message = new RequestMessage(command, ClientAddress);
    	System.out.println("Request message read: " + req_message.OK);
    	if (req_message.OK){
    		if (req_message.RT.equalsIgnoreCase("new game")){
    			System.out.println("### Running new game");
    			return newGame(req_message);
			}
    		if (req_message.RT.equalsIgnoreCase("server status")){
    			System.out.println("### Running server status");
    			return serverStatus();
    		}
    		if (req_message.RT.equalsIgnoreCase("list games")){
    			System.out.println("### Running list games");
    			return listGames(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("game status")){
    			System.out.println("### Running game status");
    			return gameStatus(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("game board")){
    			System.out.println("### Running game board");
    			return gameCurrentBoard(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("current board")){
    			return gameCurrentBoard(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("join game")){
    			System.out.println("### Running join game " + req_message.GID);
    			return joinGame(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("delete game")){
    			System.out.println("### Running delete game " + req_message.GID);
    			return deleteGame(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("add piece")){
    			System.out.println("### Running add piece " + req_message.GID);
    			return addPiece(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("rotate piece")){
    			System.out.println("### Running rotate piece " + req_message.GID);
    			return rotatePiece(req_message);
    		}
    		if (req_message.RT.equalsIgnoreCase("move piece")){
    			System.out.println("### Running move piece " + req_message.GID);
    			return movePiece(req_message);
    		}
		}else{
		System.out.println("Command did not match.");
		return new AnswerMessage(false,AnswerMessage.OUT_OF_PATTERN,"",-1);
		//return "-1";
	}
	//return "-1";
    return new AnswerMessage(false,AnswerMessage.INVALID_REQUEST_TYPE,"",-1);
  }
	
       
    private static AnswerMessage serverStatus(){
    	String STATUS = "<SERVER STATUS>";
    	for (int cont=0;cont<=game_last_id.intValue();cont++){
    		if (GAMES.containsKey(new Integer(cont))){
    		  STATUS = STATUS + "<" + cont+":"+ GAMES.get(new Integer(cont)).getClass().toString() + ":"+((manager.Game)GAMES.get(new Integer(cont))).getGameName() + ">";
    		}
    	}
    	//return STATUS;
    	return new AnswerMessage(true,AnswerMessage.SERVER_STATUS,STATUS,-1);
    	
    }
    
    private static AnswerMessage listGames(RequestMessage rm){
    	String STATUS = "<LIST GAMES>";
    	for (int cont=0;cont<=game_last_id.intValue();cont++){
    		if (GAMES.containsKey(new Integer(cont))){
    		   manager.Game TEMP = (manager.Game)GAMES.get(new Integer(cont));
    		   if ((rm.GT.length() <= 1) || rm.GT.equalsIgnoreCase(TEMP.GAME_TYPE) || rm.GT.equalsIgnoreCase("all")){	   
    		     String this_game ="<GAME>";
    		     this_game +=	"<GID>"+cont+"</GID>";
    		     this_game += "<GT>"+ TEMP.GAME_TYPE+"</GT>";
    		     this_game += "<ENDED>"+ TEMP.gameEnded()+"</ENDED>";
    		     this_game += "<WP>"+ TEMP.WAITING_PLAYERS+"</WP>";
    		     this_game += "<#PLAYERS>"+ TEMP.PLAYERS.size()+"</#PLAYERS>";
    		     this_game += "</GAME>";
    			//STATUS = STATUS + "<" + cont+":"+ GAMES.get(new Integer(cont)).getClass().toString() + ":"+((Manager.game)GAMES.get(new Integer(cont))).getGameName() + ">";
    		     STATUS+=this_game;
    		   }
    		}
    		
    	}
    	STATUS += "</LIST GAMES>";
    	return new AnswerMessage(true,AnswerMessage.GAME_LIST,STATUS,-1);
    }

    
    private static AnswerMessage gameStatus(RequestMessage message){
    	//System.out.println("Try to execute game status: " + message.GID);
    	int gameID = message.GID;
    	//System.out.println("Executing GAME STATUS: " + gameID);
    	if (GAMES.containsKey(new Integer(gameID))){
    		String STATUS = "<GAME STATUS>";
    		STATUS += ((manager.Game)GAMES.get(new Integer(gameID))).getStatus();
    		return new AnswerMessage(true,AnswerMessage.GAME_STATUS,STATUS,gameID);
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    }

    private static AnswerMessage joinGame(RequestMessage message){
    	int gameID = message.GID;
    	AddressPort ClientAddress = message.REQUESTER_ADDRESS;
    	System.out.print("JOIN GAME steps: 1");
    	if (GAMES.containsKey(new Integer(gameID))){
    		System.out.print(" 2");
    		if ((manager.Game)GAMES.get(new Integer(gameID)) != null){    			  
    			System.out.print(" 3"); 
    			//if(((List)GAMES_PLAYERS.get(new Integer(gameID))).size() < ((Manager.game)GAMES.get(new Integer(gameID))).getMaxPlayers()){
    			if(((manager.Game)GAMES.get(new Integer(gameID))).getPlayers().size() < ((manager.Game)GAMES.get(new Integer(gameID))).getMaxPlayers()){
    				System.out.print(" 4");
    				if (((manager.Game)GAMES.get(new Integer(gameID))).WAITING_PLAYERS){
    					System.out.print(" 5");
    					if (((manager.Game)GAMES.get(new Integer(gameID))).joinGame(message.PLAYER)){
    					  ((List)GAMES_PLAYERS.get(new Integer(gameID))).add(ClientAddress);
       				  	
    					  List TempPlayers = ((manager.Game)GAMES.get(new Integer(gameID))).getPlayers();
       				  	  String StringPlayers = "";
       				  	  int PlayerNumber = TempPlayers.size();
           	    		  String PlayerTag ="";
	       				  if (PlayerNumber == 1){
	       	    			  PlayerTag = AnswerMessage.PLAYER1;
	       	    		  }
	       				  if (PlayerNumber == 2){
       	    				  PlayerTag = AnswerMessage.PLAYER2;
       	    			  }
       				  	  if (TempPlayers != null){
         				  		for (int cont=0;cont<TempPlayers.size();cont++){
         				  			StringPlayers += ((Integer)TempPlayers.get(cont)).intValue() + ";"; 
         				  		}
         				  	}
       	  			  	  System.out.println("");
       		  		  	  System.out.println("Number of players: " + ((List)GAMES_PLAYERS.get(new Integer(gameID))).size());
       				      return new AnswerMessage(true,AnswerMessage.JOIN_GAME,"<GID>"+gameID+"</GID><GN>"+((manager.Game)GAMES.get(new Integer(gameID))).getGameName()+"</GN><GT>"+((manager.Game)GAMES.get(new Integer(gameID))).getGameType()+"</GT><LINES>"+((manager.Game)GAMES.get(new Integer(gameID))).getLines()+"</LINES><COLUMNS>"+((manager.Game)GAMES.get(new Integer(gameID))).getColumns()+"</COLUMNS><PLAYERS>"+StringPlayers+"</PLAYERS>"+PlayerTag,gameID);
    					}else{
        					return new AnswerMessage(false,AnswerMessage.CANNOT_JOIN_GAME,"<GID>"+gameID+"</GID><REASON>YOU ARE ALREADY IN THE GAME</REASON>",gameID);
    					}
    				}else{
    					return new AnswerMessage(false,AnswerMessage.CANNOT_JOIN_GAME,"<GID>"+gameID+"</GID><REASON>THE GAME IS NOT WAITING FOR PLAYERS</REASON>",gameID);
    				}
    			}else{
    				return new AnswerMessage(false,AnswerMessage.CANNOT_JOIN_GAME,"<GID>"+gameID+"</GID><REASON>THE GAME IS FULL OF PLAYERS</REASON>",gameID);
    			}
    		}else{
    			return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);	
    		}
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    }

    
    private static AnswerMessage deleteGame(RequestMessage message){
    	int gameID = message.GID;
    	if (GAMES.containsKey(new Integer(gameID))){
    		GAMES.remove(new Integer(gameID));
    		return new AnswerMessage(false,AnswerMessage.DELETE_GAME,"<GID>"+gameID+"</GID>",gameID);
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    }
    
    
    private static AnswerMessage addPiece(RequestMessage message){
    	int gameID = message.GID;
    	if (GAMES.containsKey(new Integer(gameID))){
    		int x = message.X;
    		int y = message.Y;
    		int Player = message.PLAYER;
    		
    		if (((manager.Game)GAMES.get(new Integer(gameID))).add(x,y,Player)){
    			return new AnswerMessage(true,AnswerMessage.PIECE_ADDED,"<GID>"+gameID+"</GID><X>"+message.X+"</X><Y>"+message.Y+"</Y>",gameID);
    			//return "<PIECE ADDED>";
    		}else{
    			System.out.println("Adding piece: " + message.X +"," + message.Y + "player " + message.PLAYER);
    			System.out.println("Game Actual Player: " + ((manager.Game)GAMES.get(new Integer(gameID))).getActualPlayer());
    			return new AnswerMessage(false,AnswerMessage.CANNOT_ADD_PIECE,"<GID>"+gameID+"</GID><X>"+message.X+"</X><Y>"+message.Y+"</Y><REASON>UNKNOWN REASON</REASON>",gameID);
    		}
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    	//return "-1";
    }


    private static AnswerMessage rotatePiece(RequestMessage message){
    	int gameID = message.GID;
    	if (GAMES.containsKey(new Integer(gameID))){
    		int x = message.X;
    		int y = message.Y;
    		String sense = message.SENSE;
    		int Player = message.PLAYER;
    		System.out.println("Rotating piece: " + message.X +"," + message.Y + "player " + message.PLAYER + " sense:" + sense);
    		if (((manager.Game)GAMES.get(new Integer(gameID))).rotate(x,y,sense,Player)){
    			return new AnswerMessage(true,AnswerMessage.PIECE_ROTATED,"<GID>"+gameID+"</GID><X>"+message.X+"</X><Y>"+message.Y+"</Y><SENSE>"+sense+"</SENSE>",gameID);
    			//return "<PIECE ROTATED>";
    		}else{
    			return new AnswerMessage(false,AnswerMessage.CANNOT_ROTATE_PIECE,"<GID>"+gameID+"</GID><X>"+message.X+"</X><Y>"+message.Y+"</Y><SENSE>"+sense+"</SENSE><REASON>UNKNOWN REASON</REASON>",gameID);
    		}
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    }
    
    private static AnswerMessage movePiece(RequestMessage message){
    	int gameID = message.GID;
    	if (GAMES.containsKey(new Integer(gameID))){
    		int x = message.X;
    		int y = message.Y;
    		int x2 = message.X2;
    		int y2 = message.Y2;
    		int Player = message.PLAYER;
    		System.out.println("Moving piece:  FROM " + message.X +"," + message.Y + "  TO " + message.X2+"," + message.Y2 + " Player " + message.PLAYER);
    		if (((manager.Game)GAMES.get(new Integer(gameID))).move(x,y,x2,y2,Player)){
    			return new AnswerMessage(true,AnswerMessage.PIECE_MOVED,"<GID>"+gameID+"</GID><X>"+message.X+"</X><Y>"+message.Y+"</Y><X2>"+message.X2+"</X2><Y2>"+message.Y2+"</Y2>",gameID);
    		}else{
    			return new AnswerMessage(false,AnswerMessage.CANNOT_MOVE_PIECE,"<GID>"+gameID+"</GID><X>"+message.X+"</X><Y>"+message.Y+"</Y><X2>"+message.X2+"</X2><Y2>"+message.Y2+"</Y2><REASON>UNKNOWN REASON</REASON>",gameID);
    		}
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    }
    

    
    private static AnswerMessage gameCurrentBoard(RequestMessage message){
    	int gameID = message.GID;
    	if (GAMES.containsKey(new Integer(gameID))){
    		String STATUS = "<GAME CURRENT BOARD>";
    		STATUS += ((manager.Game)GAMES.get(new Integer(gameID))).currentBoard();
    		STATUS += "</GAME CURRENT BOARD><LINES>"+((manager.Game)GAMES.get(new Integer(gameID))).getLines()+"</LINES><COLUMNS>"+((manager.Game)GAMES.get(new Integer(gameID))).getColumns()+"</COLUMNS><GAME INFO>"+((manager.Game)GAMES.get(new Integer(gameID))).LAST_MESSAGE+"</GAME INFO>";
    		return new AnswerMessage(true,AnswerMessage.GAME_BOARD,"<GID>"+gameID+"</GID>"+STATUS,gameID);
    	}else{
    		return new AnswerMessage(false,AnswerMessage.GAME_DOES_NOT_EXIST,"<GID>"+gameID+"</GID>",gameID);
    	}
    }
    
    
    private static AnswerMessage newGame(RequestMessage message){    	
    	AddressPort ClientAddress = message.REQUESTER_ADDRESS;
    	manager.Game NEW_GAME = null;
    	String game_type = message.GT;
    	String game_name = message.GN;
    	int player1 = message.PLAYER;
    	    	
    	System.out.println("NEW GAME: game type " + game_type+ ", game name: " + game_name + ", player id:" + player1);
    	if (game_type.equalsIgnoreCase("checkers")){
   			if (player1 < 0){
   				NEW_GAME = new manager.CheckersGame(game_name);	
   			}else{
   				NEW_GAME = new manager.CheckersGame(game_name,player1);
   			}
    	}else if ((game_type.equalsIgnoreCase("tictactoe")) || (game_type.equalsIgnoreCase("tic tac toe"))){
    		if (player1 < 0){
   				NEW_GAME = new manager.TictactoeGame(game_name);	
   			}else{
   				NEW_GAME = new manager.TictactoeGame(game_name,player1);
    		}
    	}else if (game_type.equalsIgnoreCase("deflexion")){
   			if (player1 < 0){
   				NEW_GAME = new manager.DeflexionGame(game_name);	
   			}else{
   				NEW_GAME = new manager.DeflexionGame(game_name,player1);
    		}
		}else if (game_type.equalsIgnoreCase("chess")){
			if (player1 < 0){
				NEW_GAME = new manager.ChessGame(game_name);	
			}else{
				NEW_GAME = new manager.ChessGame(game_name,player1);
			}
		}

    	if (NEW_GAME != null){
    		game_last_id = new Integer(1+game_last_id.intValue());  		    		
    		NEW_GAME.GAME_ENDED = false;
    		GAMES.put(game_last_id,NEW_GAME);
    		//System.out.println("New game " + game_type);
    		System.out.println("   => NEW GAME CREATED "+ game_last_id.intValue() + " players:  " + ((manager.Game)(GAMES.get(game_last_id))).getPlayers().size());
    		GAMES_PLAYERS.put(game_last_id,new ArrayList());
			if (player1 >=0){
				((List)(GAMES_PLAYERS.get(game_last_id))).add(0,ClientAddress);
				System.out.println("SIZE: " + ((List)(GAMES_PLAYERS.get(game_last_id))).size());
				//.add(0,ClientAddress);
				
			}
			
			return new AnswerMessage(true,AnswerMessage.NEW_GAME,"<GID>"+game_last_id+"</GID><LINES>"+((manager.Game)GAMES.get(game_last_id)).getLines()+"</LINES><COLUMNS>"+((manager.Game)GAMES.get(game_last_id)).getColumns()+"</COLUMNS>",game_last_id.intValue());
			//return "<NEW GAME:"+game_last_id+"><LINES:"+((Manager.game)GAMES.get(game_last_id)).getLines()+"><COLUMNS:"+((Manager.game)GAMES.get(game_last_id)).getColumns()+">";   		S
    	}
    	return new AnswerMessage(false,AnswerMessage.NEW_GAME,"",-1);
    	//return "-1";    	
    }
    
    
    static class DatagramEchoService extends Thread
    {
	DatagramSocket ds;

	DatagramEchoService(int port) 
		throws IOException 
	{
	    super("datagram service");
	    ds = new DatagramSocket(port);
	}
	private static final int LEN = 65530;
	public void run() {
	    DatagramPacket dp;
	    dp = new DatagramPacket(new byte[LEN], LEN);
	    String answer_message = "";
	    while (true) {
		try {
		    dp.setLength(LEN);
		    ds.receive(dp);
		    //System.out.println(dp.getLength());
		    String message = new String(dp.getData(), 0, dp.getLength());
		    System.out.println(message);
		    //ds.send(dp);
		    byte [] response;
		    //System.out.println(dp.getAddress().toString() + " " + dp.getPort());
		    AddressPort ClientAddress = new AddressPort(dp.getAddress(),dp.getPort());
		    try {
		    	System.out.println("Received: "+message);
		  /*  	String result = executeRequest(message,ClientAddress); 
		    if (result != "-1"){
		    	response = new String(result + "<:>" + message + "<:>OK").getBytes();
		    }else{
		    	 response = new String(result + "<:>" + message + "<:>NOK").getBytes();
		    }
		    */
		    	RequestMessage req_message = new RequestMessage(message, ClientAddress);
		    	AnswerMessage am = executeRequest(req_message);
		    	if (am.GID >= 0){
		    	  am.GAME_GENERAL_STATUS = gameGeneralStatus(new Integer(am.GID),req_message.PLAYER);
		    	}else{
		    		am.GAME_GENERAL_STATUS = AnswerMessage.GGS_NOTHING_TO_BE_DONE;
		    	}
		    	answer_message = am.message2string();
		    	
		    	response = answer_message.getBytes();
		    }catch(Exception e){
		    	System.err.println("ERROR: "+message + "  " + e.toString());
		    	//response = new String("-1" + "<:>" + message + "<:>NOK").getBytes();
		    	answer_message =  new AnswerMessage(false,AnswerMessage.UNKNOWN_ERROR,"",-1).message2string();
		    	response = answer_message.getBytes();
		    }
		    
		     System.out.println("Sending: " + answer_message);
	    	 DatagramPacket resp_packet = new DatagramPacket(response, response.length, dp.getAddress(), dp.getPort());
	    	 ds.send(resp_packet);
		}
		catch (IOException e) {
		    System.err.println("Exception on UDP service: " + e);
		}
	    }
	}
	
	private String gameGeneralStatus(Integer gameID,int player) {
		//TODO
	   	if (GAMES.containsKey(gameID)){
    		if ((manager.Game)GAMES.get(gameID) != null){  
    			manager.Game g = (manager.Game)GAMES.get(gameID);
    			if (g.GAME_ENDED){
    				if (g.WINNER < 0){
    					return AnswerMessage.GGS_GAME_ENDED+AnswerMessage.DRAWN_GAME+"<GAME LAST MESSAGE>"+g.LAST_MESSAGE+"</GAME LAST MESSAGE>";
    				}else{
    					return AnswerMessage.GGS_GAME_ENDED+"<WINNER>"+g.WINNER+"</WINNER>"+"<GAME LAST MESSAGE>"+g.LAST_MESSAGE+"</GAME LAST MESSAGE>";
    				}
    			}
    			if (g.WAITING_PLAYERS){
    				return AnswerMessage.GGS_WAITING_FOR_PLAYERS;
    			}
    			String PlayerTag = "";
    			if (g.getActualPlayer() == ((Integer)g.PLAYERS.get(0)).intValue()){
    				PlayerTag = AnswerMessage.PLAYER1;
    			}
    			if (g.getActualPlayer() == ((Integer)g.PLAYERS.get(1)).intValue()){
    				PlayerTag = AnswerMessage.PLAYER2;
    			}
    			if (g.getActualPlayer() == player){
    				return AnswerMessage.GGS_YOUR_TURN+PlayerTag;
    			}
    			if (g.getActualPlayer() != player){
    				return AnswerMessage.GGS_ADVERSARY_TURN+PlayerTag;
    			}
    			return "UNKNOW STATUS";
    		}
    		return AnswerMessage.GGS_GAME_DOES_NOT_EXIST;
	   	}
		return null;
	}
  }


    
    static class StreamEchoServer extends Thread {
	private Socket s;

	public StreamEchoServer(Socket cs) {
	    super("server");
	    s = cs;
	}
	public void run() {
	    InputStream is = null;
	    OutputStream os = null;
	    try {
		is = s.getInputStream();
		os = s.getOutputStream();
		byte [] buf = new byte[1024];
		int cc;
		do {
		    cc = is.read(buf);
		    if (cc > 0) os.write(buf, 0, cc);
		} while (cc >= 0);
	    }
	    catch (IOException ie) { }
	    finally {
		try { is.close(); } catch (IOException ie) { }
		try { os.close(); } catch (IOException ie) { }
		try { s.close(); } catch (IOException ie) { }
	    }
	}
    }

    static class StreamEchoService extends Thread
    {
	ServerSocket ss;

	StreamEchoService(int port) 
		throws IOException
	{
	    super("stream service");
	    ss = new ServerSocket(port);
	}

	public void run() {
	    Socket cs;
	    while(true) {
		try {
		    cs = ss.accept();
		    (new StreamEchoServer(cs)).start();
		}
		catch (IOException ie) { break; }
	    }
	}
  }

    
    public void start() {
    	tcpThread.start();
    	udpThread.start();
    	return;
        }


    
    /**
     * Test main - test out the echo server on its
     * default port, or on a port specified by args[0].
     */
    public static void main(String args[]) {
	if (args.length > 0) {
	    try {
		int prt = Integer.parseInt(args[0]);
		if (prt > 0 && prt < 65535)
		    ECHO_PORT = prt;
	    } 
	    catch (Exception e) { }
	}
	System.out.println("Starting echo service on port " + ECHO_PORT);
	GameServer ds = new GameServer();
	ds.start();
	return;
    }
}
