package netGameManager;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class contains the basic attributes for a message that is executed in several steps in the {@link GameServer}.
 * The field REQUEST came from some ({@link GameClient}) and the GameServer will try to answer the request.
 * The answer text is stored in the field RESPONSE and the field OK says if the request was or not attended.
 * 
 * @author Luciano Antonio Digiampietri
 * @version 0.5 beta - 03/15/2006
 * 
 * @param REQUEST (String) - request from the client;
 * @param RESPONSE (String) - response from the server;
 * @param OK (boolean) - status of the request (attended or not);
 * @see   GameClient
 * @see   GameServer
 */

/*
*   LIST OF GAMES:	<REQUEST><RT>LIST GAMES</RT><GT>[all|checkers|deflexion|...]</GT><GS>[all|waiting_players]</GS></REQUEST>
*   NEW GAME:		<REQUEST><RT>NEW GAME</RT><GT>[checkers|deflexion|...]CHECKERS</GT>[<GAME_NAME>[string game name]</GAME_NAME>][<PLAYER>[|player1_id]</PLAYER>]</REQUEST>
*   JOIN GAME:	   	<REQUEST><RT>JOIN GAME</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER></REQUEST>   
*   DELETE GAME:	<REQUEST><RT>DELETE GAME</RT><GID>[game_id]</GID></REQUEST>
*   ADD PIECE:    	<REQUEST><RT>ADD PIECE</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER><X>[x value]</X><Y>[y value]</Y></REQUEST>
*   MOVE PIECE:     <REQUEST><RT>MOVE PIECE</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER><X>[from x1]</X><Y>[from y1]</Y><X2>[to x2]</X2><Y2>[to y2]</Y2></REQUEST>
*   ROTATE PIECE:	<REQUEST><RT>ROTATE PIECE</RT><GID>[game_id]</GID><PLAYER>[player_id]</PLAYER><X>[x value]</X><Y>[y value]</Y><SENSE>[CLOCKWISE|ANTICLOCKWISE]</SENSE></REQUEST>
*/

public class RequestMessage {
	public boolean OK = false;
	public String RT = "";
	public String GT = "";
	public String GN = "";
	public int GID = -1;
	public int PLAYER = -1;
	public String SENSE = "";
	public int X = -1;
	public int Y = -1;
	public int X2 = -1;
	public int Y2 = -1;
	public String REQUEST = null;
	public AddressPort REQUESTER_ADDRESS = null;
	public RequestMessage(String request, AddressPort address){
		REQUEST = request;
    	OK = false;
		Pattern is_request = Pattern.compile("<REQUEST>(.+)</REQUEST>",Pattern.CASE_INSENSITIVE);
    	Matcher m_class = is_request.matcher(request);
		if (m_class.find()) {
		    request = m_class.group(1);
			Pattern kind_of_request = Pattern.compile("(.+)<(.+)>(.+)<.+>",Pattern.CASE_INSENSITIVE);
		    m_class = kind_of_request.matcher(request);
		    System.out.println(request);
		    while (m_class.find()) {
		    	String type = m_class.group(2);
		    	String value = m_class.group(3);
		    	request = m_class.group(1);
		    	m_class = kind_of_request.matcher(request);
		    	if (!(update_data(type, value))){
		    		System.err.println("Field not known: " + type + " => " + value);
		    	}
		    }
		    kind_of_request = Pattern.compile("<(.+)>(.+)<(.+)>",Pattern.CASE_INSENSITIVE);
		    m_class = kind_of_request.matcher(request);
		    if (m_class.find()) {
		    	String type = m_class.group(1);
		    	String value = m_class.group(2);
		    	//System.out.println(type + " => " + value);
		    	if (!(update_data(type, value))){
		    		System.err.println("Field not known: " + type + " => " + value);
		    	}
		    	OK = true;
		    }
		}
	}
	
	private boolean update_data(String type, String value) {
		if (type.equalsIgnoreCase("RT")){
			RT = value;
		}else
		if (type.equalsIgnoreCase("GT")){
			GT = value;
		}else
		if (type.equalsIgnoreCase("GN")){
			GN = value;
		}else
		if (type.equalsIgnoreCase("GID")){
			GID = Integer.parseInt(value);
		}else
		if (type.equalsIgnoreCase("PLAYER")){
			PLAYER = Integer.parseInt(value);
		}else
		if (type.equalsIgnoreCase("SENSE")){
			SENSE = value;
		}else
		if (type.equalsIgnoreCase("X")){
			X = Integer.parseInt(value);
		}else
		if (type.equalsIgnoreCase("Y")){
			Y = Integer.parseInt(value);
		}else
		if (type.equalsIgnoreCase("X2")){
			X2 = Integer.parseInt(value);
		}else
		if (type.equalsIgnoreCase("Y2")){
			Y2 = Integer.parseInt(value);
		}else{
			return false;
		}
		return true;
	}	
}
